/**
 * Created by sinrst on 9/13/2016.
 */
jQuery.sap.declare("sas.dabt.configurations.proxy.ConfigurationProxy");
jQuery.sap.require("sas.dabt.common.util.ErrorUtil");
sas.dabt.configurations.proxy.ConfigurationProxy = {
    SIZE: 20000,
    getConfigurationLinks: function () {
        return sas.ajax({
            type: "GET",
            url: "rest/configuration",
            dataType: "json",
            async: true,
            cache: false,
            success: function (data, textStatus, jqXHR) {
            },
            error: function (jqXHR) {
                sas.dabt.common.util.ErrorUtil.handleAjaxRequestError(jqXHR);
            },
            complete: function (jqXHR) {
            }
        });
    },

    deploySubjectGroups: function (uri) {
        var deferred = $.Deferred();
        sas.ajax(
            {
                type: 'POST',
                //url: "rest" + uri,
                url: uri,
                dataType: "json",
                accepts: {json: "application/json;charset=utf-8"},
                contentType: "application/json"
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },
    updateExternalCode: function (uri, oData,lastModified) {
        return sas.ajax({
            type:       "PUT",
            //url:        "rest"+uri,
            url: uri,
            contentType:   "application/vnd.sas.dabt.code.external+json",
            beforeSend: function (request) {
                request.setRequestHeader("If-Unmodified-Since", lastModified);
            },
            async:      true,
            cache:      false,
            data: JSON.stringify(oData),
            success: function (data, textStatus, jqXHR) {
                if(jqXHR.getResponseHeader("Last-Modified")){
                    data.lastModifiedHeader = jqXHR.getResponseHeader("Last-Modified");
                }
            },
            complete: function(jqXHR) {
                // completeHandler(jqXHR);
            }
        });
    },
    createExternalCode: function (uri, oData) {
        var deferred = $.Deferred();
        sas.ajax(
            {
                type: 'POST',
                //url: "rest" + uri,
                url: uri,
                dataType: "json",
                accepts: {json: "application/vnd.sas.dabt.code.external+json;charset=utf-8"},
                contentType: "application/json",
                data: JSON.stringify(oData)
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },
    getSystemGeneratedVariables: function (uri) {
        var deferred = $.Deferred();
        sas.ajax(
            {
                type: "GET",
                //url: "rest" + uri + "?filter=eq(applicableProcessType, DER_VAR)&sortBy=name&start=0&limit=" + this.SIZE,
                url: uri + "?filter=eq(applicableProcessType, DER_VAR)&sortBy=name&start=0&limit=" + this.SIZE,
                dataType: "json",
                async: true,
                cache: false,
                accepts: {json: "application/json;charset=utf-8"},
                contentType: "application/json"
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },
    getActiveSubjectOfAnalyses: function (uri) {
        var deferred = $.Deferred();
        sas.ajax(
            {
                type: "GET",
                //url: "rest" + uri + "?filter=eq(enabled,true)&sortBy=name&start=0&limit=" + this.SIZE,
                url: uri + "?filter=eq(enabled,true)&sortBy=name&start=0&limit=" + this.SIZE,
                dataType: "json",
                async: true,
                cache: false,
                accepts: {json: "application/json;charset=utf-8"},
                contentType: "application/json"
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },
    createExternalVariable: function (uri, oData) {
        var deferred = $.Deferred();
        sas.ajax(
            {
                type: 'POST',
                //url: "rest" + uri,
                url: uri,
                dataType: "json",
                accepts: {json: "application/vnd.sas.dabt.code.external+json;charset=utf-8"},
                contentType: "application/json",
                data: JSON.stringify(oData)
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },
    /**
     * Save edited data source.
     */
    updateDataSource: function (uri, oData, lastModified) {
        return sas.ajax({
            type:       "PUT",
           // url:        "rest"+uri,
            url: uri,
            contentType: "application/json",
            beforeSend: function (request) {
                request.setRequestHeader("If-Unmodified-Since", lastModified);
            },
            async:      true,
            cache:      false,
            data: JSON.stringify(oData),
            success: function (data, textStatus, jqXHR) {
                if(jqXHR.getResponseHeader("Last-Modified")){
                    data.lastModifiedHeader = jqXHR.getResponseHeader("Last-Modified");
                }
            },
            complete: function(jqXHR) {
                // completeHandler(jqXHR);
            }
        });
    },
    updateExternalVariable: function (uri, oData, lastModified) {
        return sas.ajax({
            type:       "PUT",
            //url:        "rest"+uri,
            url: uri,
            contentType:   "application/vnd.sas.dabt.variable.external+json",
            beforeSend: function (request) {
                request.setRequestHeader("If-Unmodified-Since", lastModified);
            },
            async:      true,
            cache:      false,
            data: JSON.stringify(oData),
            success: function (data, textStatus, jqXHR) {
                if(jqXHR.getResponseHeader("Last-Modified")){
                    data.lastModifiedHeader = jqXHR.getResponseHeader("Last-Modified");
                }
            },
            complete: function(jqXHR) {
                // completeHandler(jqXHR);
            }
        });
    },
    getDataTypeOptions: function (uri) {
        var deferred = $.Deferred();
        sas.ajax(
            {
                type: "GET",
                //url: "rest" + uri + "?sortBy=name&start=0&limit=" + this.SIZE,
                url: uri + "?sortBy=name&start=0&limit=" + this.SIZE,
                dataType: "json",
                async: true,
                cache: false,
                accepts: {json: "application/json;charset=utf-8"},
                contentType: "application/json"
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },
    getPurposes: function(url){
        return this.getDataTypeOptions(url);
    },
    getData: function (uri) {
        return sas.ajax({
            type: "GET",
            //url: "rest" + uri,
            url: uri,
            dataType: "json",
            async: true,
            cache: false,
            success: function (data, textStatus, jqXHR) {
                if (jqXHR.getResponseHeader("Last-Modified")) {
                    data.lastModifiedHeader = jqXHR.getResponseHeader("Last-Modified");
                }
            },
            error: function (jqXHR) {
                sas.dabt.common.util.ErrorUtil.handleAjaxRequestError(jqXHR);
            },
            complete: function (jqXHR) {
            }
        });
    },
    deleteObject: function(uri,oData){
        var deferred = $.Deferred();
        sas.ajax(
            {
                type: 'POST',
                //url: "rest" + uri,
                url: uri,
                dataType: "json",
                accepts: {json: "application/vnd.sas.dabt.code.external+json;charset=utf-8"},
                contentType: "application/json",
                data: JSON.stringify(oData)
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },
    
    /**
     * Add the data sources
     */
    addDataSources : function(uri, oData) {
        return sas.ajax({
            type: "POST",
            //url: "rest" + uri,
            url: uri,
            data: JSON.stringify(oData),
            contentType: "application/vnd.sas.collection+json",
            async: true,
            cache: false,
            success: function (data, textStatus, jqXHR) {
                
            },
            error: function (jqXHR) {
                //sas.dabt.common.util.ErrorUtil.handleAjaxRequestError(jqXHR);
            },
            complete: function (jqXHR) {
            }
        });
        
    },

    /**
     * Get the time frequencies(e.g. Month, Day etc.) for the data sources.
     */
    getTimeFrequencies: function () {
        return sas.ajax({
            type: "GET",
            //url: "rest" + sas.dabt.common.util.JSONUtil.getUriByRel(sas.dabt.coreConfigJson.links, "timeFrequencies"),
            url: sas.dabt.common.util.JSONUtil.getUriByRel(sas.dabt.coreConfigJson.links, "timeFrequencies"),
            data: "filter=eq(applicableFunctionalities.type,'abtAgg')", // <-- filter query
            dataType: "json",
            async: true,
            cache: false,
            success: function (data, textStatus, jqXHR) {
                
            },
            error: function (jqXHR) {
                sas.dabt.common.util.ErrorUtil.handleAjaxRequestError(jqXHR);
            },
            complete: function (jqXHR) {
            }
        });
        
    },
    
    /**
     * Get the Field data types(e.g. Numeric, Character, Date, Datetime etc.) for the data sources.
     */
    getFieldDataTypes: function () {
        return sas.ajax({
            type: "GET",
            //url: "rest" + sas.dabt.common.util.JSONUtil.getUriByRel(sas.dabt.coreConfigJson.links, "fieldDataTypes"),
            url: sas.dabt.common.util.JSONUtil.getUriByRel(sas.dabt.coreConfigJson.links, "fieldDataTypes"),
            contentType: "application/vnd.sas.collection+json",
            dataType: "json",
            async: true,
            cache: false,
            success: function (data, textStatus, jqXHR) {
                
            },
            error: function (jqXHR) {
                sas.dabt.common.util.ErrorUtil.handleAjaxRequestError(jqXHR);
            },
            complete: function (jqXHR) {
            }
        });
    },
    
    /**
     * Get the Field column types(e.g. Measure, Dimensional Attribute, Date, Key etc.) for the data sources.
     */
    getFieldTypes: function () {
        return sas.ajax({
            type: "GET",
            //url: "rest" + sas.dabt.common.util.JSONUtil.getUriByRel(sas.dabt.coreConfigJson.links, "fieldTypes"),
            url: sas.dabt.common.util.JSONUtil.getUriByRel(sas.dabt.coreConfigJson.links, "fieldTypes"),
            contentType: "application/vnd.sas.collection+json",
            dataType: "json",
            async: true,
            cache: false,
            success: function (data, textStatus, jqXHR) {
                
            },
            error: function (jqXHR) {
                sas.dabt.common.util.ErrorUtil.handleAjaxRequestError(jqXHR);
            },
            complete: function (jqXHR) {
            }
        });
    },
    
    /**
     * Get the data source applicable usage of the variable.
     */
    getApplicableUsageOfVariable: function (uri, variableType) {
        return sas.ajax({
            type: "GET",
            //url: "rest" + uri,
            url: uri,
            data: "start=0&limit=" + this.SIZE + "&filter=eq(applicableUsages.type, " + variableType + ")", // <-- filter query
            dataType: "json",
            async: true,
            cache: false,
            success: function (data, textStatus, jqXHR) {
                
            },
            error: function (jqXHR) {
                sas.dabt.common.util.ErrorUtil.handleAjaxRequestError(jqXHR);
            },
            complete: function (jqXHR) {
            }
        });
        
    },
    
    /**
     * Get the data source applicable usage of the variable.
     */
    getAvailableColumnToConfigure: function (uri) {
        return sas.ajax({
            type: "GET",
            //url: "rest" + uri,
            url: uri,
            data: "start=0&limit=" + this.SIZE + "&filter=or(eq(dataType.uniqueCode, 'DTT'), eq(dataType.uniqueCode, 'DAT'))", // <-- filter query
            dataType: "json",
            async: true,
            cache: false,
            success: function (data, textStatus, jqXHR) {
                
            },
            error: function (jqXHR) {
                sas.dabt.common.util.ErrorUtil.handleAjaxRequestError(jqXHR);
            },
            complete: function (jqXHR) {
            }
        });
        
    },
    
    /**
     * Add the specified columns
     */
    addColumns : function(uri, oData) {
        return sas.ajax({
            type: "POST",
            //url: "rest" + uri,
            url: uri,
            data: JSON.stringify(oData),
            contentType: "application/vnd.sas.collection+json",
            async: true,
            cache: false,
            success: function (data, textStatus, jqXHR) {
                
            },
            error: function (jqXHR) {
                //sas.dabt.common.util.ErrorUtil.handleAjaxRequestError(jqXHR);
            },
            complete: function (jqXHR) {
            }
        });
        
    },
    
    /**
     * Add the specified column values
     */
    addValues : function(uri, oData) {
        return sas.ajax({
            type: "POST",
            //url: "rest" + uri,
            url: uri,
            data: JSON.stringify(oData),
            contentType: "application/vnd.sas.collection+json",
            async: true,
            cache: false,
            success: function (data, textStatus, jqXHR) {
                
            },
            error: function (jqXHR) {
                //sas.dabt.common.util.ErrorUtil.handleAjaxRequestError(jqXHR);
            },
            complete: function (jqXHR) {
            }
        });
        
    },
    
    /**
     * Sync data source table with information map.
     */
    syncInformationMap: function(uri,oData){
    	console.info("calling ConfigurationProxy.syncInformationMap");
    	return sas.ajax({
            type:       "POST",
           // url:        "rest"+uri,
            url: uri,
            contentType: "application/vnd.sas.selection+json",
            accepts: {json: "application/vnd.sas.selection+json;charset=utf-8"},
            dataType: "json",
            data: JSON.stringify(oData),
            success: function (data, textStatus, result) {
            },
            complete: function(result) {
            }
        });
    },
    /**
     * Add dimensional attribute
     */
    addDimensionalAttribute : function(uri, oData) {
        return sas.ajax({
        	type: 'POST',
            //url: "rest" + uri,
        	url: uri,
            dataType: "json",
            data: JSON.stringify(oData),
            accepts: {json: "application/vnd.sas.dabt.source.table.field.value+json;charset=utf-8"},
            contentType: "application/json",
            success: function (data, textStatus, jqXHR) {// handled in controller
            },
            error: function (jqXHR) {
            },
            complete: function (jqXHR) {
            }
        });

    },
    
    /**
     * Edit a column or dimensional attribute
     */
    
    editColumn : function(uri, oData, lastModifiedDate){
    	
    	return sas.ajax({
        	type: 'PATCH',
            //url: "rest" + uri,
        	url: uri,
            dataType: "json",
            data: JSON.stringify(oData),
            accepts: {json: "application/vnd.sas.dabt.source.table.field.value+json;charset=utf-8"},
            contentType: "application/json-patch+json",
            beforeSend: function (request) {
				request.setRequestHeader("If-Unmodified-Since", lastModifiedDate); 
			},
            success: function (data, textStatus, jqXHR) {// handled in controller
            },
            error: function (jqXHR) {
            },
            complete: function (jqXHR) {
            }
        });
    	
    }

};

