jQuery.sap.declare({modName:"sas.csb.backtesting.runs.RunsStatus", type:"view"});
jQuery.sap.require("sas.hc.ui.core.mvc.JSView");
jQuery.sap.require("sas.hc.ui.table.Table");
jQuery.sap.require("sas.hc.m.SearchField");
jQuery.sap.require("sas.hc.m.Toolbar");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");
jQuery.sap.require("sas.hc.m.HelpPopup");
jQuery.sap.require("sas.hc.ui.model.PageableJSONModel");
jQuery.sap.require("sap.ui.model.Filter");
jQuery.sap.require("sas.hc.m.StandardListItem");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.Text");
jQuery.sap.require("sas.hc.ui.table.Column");
jQuery.sap.require("sas.hc.m.ZeroState");
jQuery.sap.require("sas.cscommon.util.MessageUtil");
jQuery.sap.require("sas.cscommon.util.FormatUtil");
jQuery.sap.require("sas.cscommon.util.JSONUtil");


(function(){

	sas.hc.ui.core.mvc.JSView.extend("sas.csb.backtesting.runs.RunsStatus", {

		metadata : {
			properties : {

			}
		},
		
		rb: sap.ui.getCore().getLibraryResourceBundle("sas.csb.backtesting"),

		renderer: "sas.hc.ui.core.mvc.JSViewRenderer"

	});
	
	// PRIVATE CONTANTS
	sas.csb.backtesting.runs.RunsStatus.prototype._BINDID = "items";
	sas.csb.backtesting.runs.RunsStatus.prototype._TABLE_ROW_COUNT = 10;
	sas.csb.backtesting.runs.RunsStatus.prototype._MAX_ROW_COUNT = 10;
	sas.csb.backtesting.runs.RunsStatus.prototype._MIN_COLUMN_WIDTH = 75;

	sas.csb.backtesting.runs.RunsStatus.prototype.getControllerName = function() {
		return "sas.csb.backtesting.runs.RunsStatusController";
	};

	sas.csb.backtesting.runs.RunsStatus.prototype.createContent = function(controller) {

		// Create the zero state
		this.listZeroState = this.getZeroState();

		// Create the table
		this.table = this.createTable();

		// Add columns to the table
		this.addColumnsToTable(this.table);

		// Set model to the table
		this.setModelToTable(this.table);
		
		// Adding style class
		this.table.addStyleClass("runStatusView");

		return this.table;
	};

	sas.csb.backtesting.runs.RunsStatus.prototype.createTable = function() {
		var table = new sas.hc.ui.table.Table({
			id : this.createId("listTable"),
			minColumnWidth : this._MIN_COLUMN_WIDTH,
			enableColumnReordering : false,
			showColumnVisibilityMenu : true,
			visibleRowCountMode : sas.hc.ui.table.VisibleRowCountMode.Auto,
			minAutoRowCount : 2,
			//visibleRowCount : this._TABLE_ROW_COUNT,
			//selectionBehavior : sas.hc.ui.table.SelectionBehavior.Row,
			selectionMode : sas.hc.ui.table.SelectionMode.None,
			noData: " ",
			busyIndicatorDelay: 0,
			width: "auto",
			toolbar: this.createTableHeader()
		});

		return table;
	};

	sas.csb.backtesting.runs.RunsStatus.prototype.createTableHeader = function () {
		// Adding the search field
        this.searchField = new sas.hc.m.SearchField({
             id: this.createId("searchFld"),
             placeholder: this.rb.getText("csb.backtesting.runStatus.search.placeholder.txt"),
             search: [this.onModelNameSearched, this]

         });

		// return Toolbar with the required controls
		return new sas.hc.m.Toolbar({
			content: [
			          this.searchField
		          ]
		});
	};
	
	sas.csb.backtesting.runs.RunsStatus.prototype.onModelNameSearched = function(event){
		var filters = this.getDefaultFilters();
		var value = event.getSource().getValue() || "";
		
		// Create the filter on model name
		if(value !== "") {
		    var searchFilter = new sap.ui.model.Filter("modelName", sap.ui.model.FilterOperator.Contains, value);
		    filters.push(searchFilter);
		}
		
	    var binding = this.table.getBinding("rows");
	    binding.filter(filters);
	};	
	sas.csb.backtesting.runs.RunsStatus.prototype.setModelToTable = function(table){
		var url = this.getDataUrl() + "?modelCreatedBy=" + sas.hc.ui.core.util.UserInfo.instance.userid;
		var pgSize = this.getPageSize();
		var bindId = this.getBindId();

		if (!!url) {
			var oModel = new sas.hc.ui.model.PageableJSONModel({
				url: url,
				bindId: bindId,
				pageSize: pgSize
			});
			
			oModel.attachRequestSent(this.onRequestSent, this);
			oModel.attachRequestCompleted(this.onRequestCompleted, this);
			oModel.attachRequestFailed(this.onRequestFailed, this);			
			table.setModel(oModel);			
			var sorter = this.getDefaultSorter();
			var aFilter = this.getDefaultFilters();
			table.bindRows("/" + bindId, sorter, aFilter);
		}
	};

	/**
	 * The url returned will have filter for logged-in user & active models
	 */
	sas.csb.backtesting.runs.RunsStatus.prototype.getDataUrl = function() {
		var links = sas.csb.modelSpecificationJson.links;

		if (!links) {
			return null;
		}
		var rel = "runDetails";
	    var objectsLink = sas.cscommon.util.JSONUtil.getLinkByRel(links, rel);
	    var url = objectsLink.href;

	    return url;
	};

	sas.csb.backtesting.runs.RunsStatus.prototype.getPageSize = function() {
		return 50;
	};

	sas.csb.backtesting.runs.RunsStatus.prototype.getBindId = function() {
		return this._BINDID;
	};

	sas.csb.backtesting.runs.RunsStatus.prototype.getDefaultSorter = function() {
		var sorter = new sap.ui.model.Sorter("executionStartDate", true);

		return sorter;
	};

	sas.csb.backtesting.runs.RunsStatus.prototype.getDefaultFilters = function() {
		var filters = [];
		// S1561174: Commenting out 'modelCreatedBy' filter from default filters and adding the same in url query parameter to fix hot fix. 
		/*var userInfo = sas.hc.ui.core.util.UserInfo.instance;
		
		var userIdFilter = new sap.ui.model.Filter("modelCreatedBy", sap.ui.model.FilterOperator.EQ, userInfo.userid);
		filters.push(userIdFilter);*/
		
		var activeFilter = new sap.ui.model.Filter("reportSpecificationActiveFlag", sap.ui.model.FilterOperator.EQ, true);
		filters.push(activeFilter);
		
		var reportCategoryFilter = new sap.ui.model.Filter("reportCategory", sap.ui.model.FilterOperator.EQ, "BACKTESTING");
		filters.push(reportCategoryFilter);
		
		return filters;
	};

	sas.csb.backtesting.runs.RunsStatus.prototype.onRequestSent = function () {
		this.table.setNoData(" ");
		this.table.setBusy(true);
	};

	sas.csb.backtesting.runs.RunsStatus.prototype.onRequestCompleted = function (oEvent) {
		try {
			var oModel = oEvent.getSource();
			var rowCount = oModel.totalRowCount;
			this.updateZeroState(rowCount);
		} finally {
			this.table.setBusy(false);
		}
	};

	sas.csb.backtesting.runs.RunsStatus.prototype.onRequestFailed = function (jqXHR) {
		this.table.setBusy(false);

		sas.cscommon.util.MessageUtil.showErrorMessageBox(this.rb.getText("csb.backtesting.runStatus.error.message.txt"), 
				this.rb.getText("csb.backtesting.runStatus.error.window.title"));
	};

	sas.csb.backtesting.runs.RunsStatus.prototype.addColumnsToTable = function(table) {
		var that = this;
		
		table.addColumn(new sas.hc.ui.table.Column({
			label : new sas.hc.m.Label({
				text : this.rb.getText("csb.backtesting.runStatus.modelName.header.txt")
			}),
			template : new sas.hc.m.Text({
				text : "{modelName}"
			}),
			sortProperty : "modelName",
			width:"10rem"
		}));

		table.addColumn(new sas.hc.ui.table.Column({
			label : new sas.hc.m.Label({
				text : this.rb.getText("csb.backtesting.runStatus.versions.header.txt")
			}),
			template : new sas.hc.m.Text({
				text : "{reportSpecificationName}"
			}),
			sortProperty : "reportSpecificationName",
			width:"12rem"
		}));

		table.addColumn(new sas.hc.ui.table.Column({
			label : new sas.hc.m.Label({
				text : this.rb.getText("csb.backtesting.runStatus.scoringDate.header.txt")
			}),
			template : new sas.hc.m.Text().bindProperty("text", {
				path : "scoringTimeDate",
				formatter : function(obj) {
				    // We want to show the date as is which is sent by Rest API(which is in UTC format) without considering the time zone,
				    // Considering time zone, the date may change due to time offset. So we are using the UTC date. // S1438496 & S1440995
				    return sas.cscommon.util.FormatUtil.formatDate(obj, {style: "medium"}, true);
				}
			}),
			sortProperty : "scoringTimeDate",
			width:"10rem"
		}));

		table.addColumn(new sas.hc.ui.table.Column({
			label : new sas.hc.m.Label({
				text : this.rb.getText("csb.backtesting.runStatus.dateExecuted.header.txt")
			}),
			template : new sas.hc.m.Text().bindProperty("text", {
				path : "executionStartDate",
				formatter : function(obj) {
					return sas.cscommon.util.FormatUtil.formatDateTime(obj);
				}
			}),
			sortProperty : "executionStartDate",
			width:"12rem"
		}));

		table.addColumn(new sas.hc.ui.table.Column({
			label : new sas.hc.m.Label({
				text : this.rb.getText("csb.backtesting.runStatus.status.header.txt")
			}),
			template: new sas.hc.m.StandardListItem({
				type: sap.m.ListType.Inactive,
				title: {
					path: "status",
					formatter: function (value) {
						var text = "";
						if (!!value) {
							if (value === sas.csb.app.Constants.RUN_STATUS_COMPLETED) {
								text = that.rb.getText("csb.backtesting.runStatus.satus.complete.txt");
							} else if (value === sas.csb.app.Constants.RUN_STATUS_FAILED) {
								text = that.rb.getText("csb.backtesting.runStatus.satus.failed.txt");
							} else if (value === sas.csb.app.Constants.RUN_STATUS_RUNNING) {
								text = that.rb.getText("csb.backtesting.runStatus.satus.inProcess.txt");
							}
						}
						return text;
					}
				},
				icon: {
					path: "status",
					formatter: function (value) {
						var icn = "";
						if (!!value) {
							if (value === sas.csb.app.Constants.RUN_STATUS_COMPLETED) {
								icn = sas.icons.HC.COMPLETESTATUS;
							} else if (value === sas.csb.app.Constants.RUN_STATUS_FAILED) {
								icn = sas.icons.HC.ERRORSTATUS;
							} else if (value === sas.csb.app.Constants.RUN_STATUS_RUNNING) {
								icn = sas.icons.HC.PROGRESSSTATUS;
							}
						}
						return icn;
					}
				}
			}).addStyleClass("runStatusColumnCell"),
			sortProperty : "status",
			width:"10rem"
		}));

	};

	sas.csb.backtesting.runs.RunsStatus.prototype.updateZeroState = function (rowCount) {
		if (rowCount === 0) {
			
			// Check if search is performed
			var searchStr = this.searchField.getValue();
			
			if(searchStr && searchStr.trim() !== "") {
				this.listZeroState.setText(this.rb.getText("csb.backtesting.runStatus.noDataFound.txt"));
			} else {
				this.listZeroState.setText(this.rb.getText("csb.backtesting.runStatus.noData.txt"));
			}
			
			this.table.setNoData(this.listZeroState);
		}
	};

	sas.csb.backtesting.runs.RunsStatus.prototype.getZeroState = function() {
		var zeroState = new sas.hc.m.ZeroState({
			text: this.rb.getText("csb.backtesting.runStatus.noData.txt"),
			icon: sas.icons.HC.ZEROSTATEZERO
		});

		return zeroState;
	};

	sas.csb.backtesting.runs.RunsStatus.prototype.destroy = function() {
		if (this.table) {
			this.table.destroy();
		}
		if (this.listZeroState) {
			this.listZeroState.destroy();
		}
		
		sas.hc.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
	};

})();