jQuery.sap.declare("sas.hc.ui.core.Core");

jQuery.sap.require("sas.hc.ui.core.Configuration");
jQuery.sap.require("sas.hc.ui.services.ServiceRegistry");
jQuery.sap.require("sas.hc.ui.core.BootTask");
jQuery.sap.require("sas.hc.ui.core.util");
jQuery.sap.require("sas.hc.ui.core.settings.SettingsInfo");
jQuery.sap.require("sas.hc.ui.core.settings.SettingsInfoTask");
jQuery.sap.require("sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherBootTask");
jQuery.sap.require("sas.hc.ui.core.TouchDetectionTask");
jQuery.sap.require("sas.hc.ui.core.MonkeyPatchApplySettingsTask");
jQuery.sap.require("sas.hc.ui.core.ContextMenuTask");
jQuery.sap.require("sas.hc.ui.core.ClearFocusRingTask");
jQuery.sap.require("sas.hc.ui.core.TimeoutHandlerTask");
jQuery.sap.require("sas.hc.ui.core.theme.ThemeBootTask");
jQuery.sap.require("sap.ui.core.Core");
jQuery.sap.require("sap.ui.core.RenderManager");
jQuery.sap.require("sas.hc.ui.core.UtilTask");
jQuery.sap.require("sas.hc.ui.core.util.ComponentServiceClassResolver");
jQuery.sap.require("sap.ui.core.Locale");
jQuery.sap.require("sas.hc.ui.core.util.UserInfo");
jQuery.sap.require("sas.hc.ui.core.util.UserInfoTask");
jQuery.sap.require("sas.hc.ui.core.CSRFTask");

(function(){
    /**
     *
     *
     * Core class that boots sas commons infrastructure to be accessed using sas.hc.getCore() utility api.
     * @returns {sas.hc.ui.core.Core} TODO-jsdoc
     */
    sas.hc.ui.core.Core = function(){
        // sas-namespaced logging
        sas.log = jQuery.extend($.sap.log);

        if(sas.hc.ui.getCore && sas.hc.ui.getCore()){
            return sas.hc.ui.getCore();
        }
        var that = this;
        if(sas.hc.ui.getCore === undefined){
            // Support custom styles rendering.
            var _invalidatedStyleControls={},
                renderCustomStylesTimer=undefined,
                toCSS=function(oObj){
                    var cssString,finalCssString="";
                    for(var prop in oObj){
                        if (oObj.hasOwnProperty(prop)) {
                            cssString="";
                            if(typeof oObj[prop] === 'object' && oObj[prop]){
                                cssString=JSON.stringify(oObj[prop],
                                    function(key,val){
                                        if(typeof val === 'string'){
                                            return val.replace(/,/g,'_Q_').replace(/\"/g,'');
                                        }
                                        return val;
                                    }
                                ).replace(/\"/g,'');
                                cssString=cssString.replace(/,/g,';').replace(/_Q_/g,',');
                                finalCssString += prop + cssString;
                            }
                        }
                    }
                    return finalCssString;
                },
                createStyleElm = function(id,content){
                    var styleEl = document.createElement('style');
                    styleEl.id=id;
                    styleEl.type="text/css";
                    if(typeof content === 'object'){
                        styleEl.textContent = toCSS(content);
                    }else{
                        styleEl.textContent=content;
                    }
                    return styleEl;
                },
                renderInvalidatedCustomStyles = function(){
                    sas.log.trace('Render invalidated custom styles');
                    var headElm = document.head,
                        fragment=document.createDocumentFragment(); //TODO more than 1 head elements. Issue.
                    do{
                        if(typeof renderCustomStylesTimer !== 'undefined'){
                        // clear timer.
                            jQuery.sap.clearDelayedCall(renderCustomStylesTimer);
                            renderCustomStylesTimer = undefined;
                        }

                        for(var sId in _invalidatedStyleControls){
                            if (_invalidatedStyleControls.hasOwnProperty(sId)) {
                                var oControl = _invalidatedStyleControls[sId];
                                delete _invalidatedStyleControls[sId];
                                // remove existing rules for this control if any.
                                // style tag for a control will have its id-style
                                var styleId = oControl.getId()+"-style";
                                var oldStyleElm = document.getElementById(styleId);
                                if(oldStyleElm){
                                    headElm.removeChild(oldStyleElm);
                                }
                                if(!oControl._bIsBeingDestroyed && oControl.getCustomStyles && oControl.getCustomStyles() && typeof oControl.getCustomStyles() !== 'undefined'){
                                    var renderer = oControl.getRenderer();
                                    if(typeof renderer === 'object' && typeof renderer.getCustomCSSRules === 'function'){
                                        fragment.appendChild(createStyleElm(styleId,renderer.getCustomCSSRules(oControl)));
                                    }else{
                                        sas.log.warning(oControl.toString()+' does not fully support setCustomStyles() API');
                                    }
                                }
                            }
                        }
                    }while(typeof renderCustomStylesTimer !== 'undefined');
                    // add all style elements to head.
                    headElm.appendChild(fragment);
                };

            // Proxy sap.ui.core.RenderManager.flush
            // Render custom styles before 'onAfterRendering' is called on controls.
            var fnOldFlush = sap.ui.core.RenderManager.prototype.flush;
            sap.ui.core.RenderManager.prototype.flush = function(){
                // go ahead and render custom styles.
                renderInvalidatedCustomStyles();
                fnOldFlush.apply(this,arguments);
            };

            // Proxy Control.renderControl function in Control class.
            var fnOldRenderControl = sap.ui.core.RenderManager.prototype.renderControl;
            sap.ui.core.RenderManager.prototype.renderControl = function(oCntrl){
                fnOldRenderControl.apply(this,arguments);
                if(oCntrl && oCntrl.getCustomStyles && oCntrl.getCustomStyles()){
                    sas.hc.ui.getCore().addInvalidateCustomStyles(oCntrl,true);
                }
            };

            Object.defineProperty(sas.hc.ui,'getCore',{
                enumerable:true,
                value:function(){
                    return Object.freeze({
                        registerTask:function(oTask){
                            that.registerTask.apply(that, arguments);
                        },
                        boot:function(fnCallBack){
                            that.boot.apply(that, arguments);
                        },
                        addInvalidateCustomStyles:function(oControl,bSuppressRendering){
                            jQuery.sap.assert(oControl && oControl instanceof sap.ui.core.Control, "oControl must be a sap.ui.core.Control");
                            _invalidatedStyleControls[oControl.getId()]=oControl;
                            bSuppressRendering = typeof bSuppressRendering === 'boolean' && bSuppressRendering;
                            if(!bSuppressRendering && typeof renderCustomStylesTimer === 'undefined'){
                                renderCustomStylesTimer = jQuery.sap.delayedCall(0,this,renderInvalidatedCustomStyles);
                            }
                        }
                    });
                }
            });
        }
    };

    function TasksChain(parent){
        this.tasksMap = new Map();
        this.next = undefined;
        this.parent = parent;
    };

    TasksChain.prototype.destroy = function(){
        this.tasksMap.clear();
        this.tasksMap = undefined;
        if(this.next){
            this.next.destroy();
            this.next = undefined;
        }
        this.parent = undefined;
    };

    TasksChain.prototype._isAncestor = function(node){
        while(node && node.parent){
            if (node.parent === this){
                return true;
            }
            node = node.parent;
        }
        return false;
    };

    TasksChain.prototype._findByName = function(name){
        if(this.tasksMap.get(name)){
            return this;
        }else if(this.next){
            return this.next._findByName(name);
        }
    };


    TasksChain.prototype._findParent = function(deps){
        var parentChain,
            potentialParent,
            iTaskDepName;
        while(deps.length > 0){
            iTaskDepName = deps.shift();
            potentialParent = this._findByName(iTaskDepName);
            if(potentialParent && parentChain){
                // check if potential parent is ancestor of this
                if(!potentialParent._isAncestor(parentChain)){
                    parentChain = potentialParent;
                }
            }else if(potentialParent){
                parentChain = potentialParent;
            }
        }
        return parentChain;
    };

    /**
     * Adds given task to chain depending upon its dependencies specified if any.
     * Expects specified dependency to have registered before the input task itself otherwise
     * adds the task to the default chain.
     */
    TasksChain.prototype.add = function(oTask){
        var parentChain;
        jQuery.sap.assert(oTask && oTask instanceof sas.hc.ui.core.BootTask, "oTask must be a sas.hc.ui.core.BootTask");
        jQuery.sap.assert(typeof oTask.name === "string" && oTask.name !== "", "oTask.name must be a non empty string");
        if(oTask.dependsOn){
            jQuery.sap.assert(Array.isArray(oTask.dependsOn), "oTask.dependsOn must be an array of strings");
            parentChain = this._findParent(oTask.dependsOn);
            if(parentChain){
                parentChain.next = parentChain.next || new TasksChain(parentChain);
                parentChain.next.tasksMap.set(oTask.name,oTask);
                return;
            }
        }
        this.tasksMap.set(oTask.name,oTask);
    };

    TasksChain.prototype.execute = function(){
        var that = this;

        return new Promise(function(resolve,reject){

            var asyncTasks;

            function executeNext(){
                if(that.next){
                    that.next.execute().then(resolve);
                }else{
                    resolve();
                }
            }

            that.tasksMap.forEach(function(oTask){
                sas.log.info('Executing task',oTask.name);
                if(oTask['runAsync']){
                    asyncTasks = asyncTasks || [];
                    asyncTasks.push(oTask.executeAsync.apply(oTask));
                }else{
                    oTask.execute.apply(oTask);
                }
            });

            if(asyncTasks && asyncTasks.length > 0){
                Promise.all(asyncTasks).then(executeNext);
            }else{
                executeNext();
            }
        });
    };

    sas.hc.ui.core.Core.prototype.tasks = new TasksChain();

    function registerCacheFilter(){
        //S1187387: Make sure IE does not used cached results from REST GET/HEAD calls.
        if( sap.ui.Device.browser.name === sap.ui.Device.browser.BROWSER.INTERNET_EXPLORER ) {
            //S1297772 exclude library preloads from IE bypass cache mechanism.
            var excludesPattern = sas.config.get("xx-bypassIECacheExcludes") || /.*\/library-preload\.json$/;

            jQuery.ajaxPrefilter(function( options, originalOptions, jqXHR ) {
                // Return if cache option already specified or if this is not a GET/HEAD call.
                if ( !options ||
                    (typeof options!=="object") ||
                    (options.cache!==undefined) ||
                    (options.type &&
                        (options.type.toUpperCase()!=="GET") &&
                        (options.type.toUpperCase()!=="HEAD")) ||
                    (typeof options.url === 'string' && options.url.match(excludesPattern))){
                    return;
                }

                // Set cache:false if Accept header value specified and its value contains JSON or XML.
                if( options.headers &&
                    (typeof options.headers==='object') ) {
                    for(var propertyName in options.headers) {
                        if (options.headers.hasOwnProperty(propertyName)) {
                            if ( propertyName.toUpperCase() === "ACCEPT" ) {
                                if( (typeof options.headers[propertyName]==="string") &&
                                    (options.headers[propertyName].toUpperCase().indexOf("JSON")>-1) ||
                                    (options.headers[propertyName].toUpperCase().indexOf("XML")>-1)) {
                                    options.cache=false;
                                    return;
                                }
                            }
                        }
                    }
                }
                // Return if dataType is not JSON and not XML.
                if ( options.dataType &&
                    (options.dataType.toUpperCase().indexOf("JSON")<0) &&
                    (options.dataType.toUpperCase().indexOf("XML")<0) )
                    return;

                // Set the cache option to false.
                options.cache=false;
            });
        }
    };

    // private function to register commons
    // boot task.
    sas.hc.ui.core.Core.prototype._registerCoreTask = function(oTask){
        this.tasks.add(oTask);
    };

    function configureLanguage(){
        var sapConf = sap.ui.getCore().getConfiguration(),
            confLang = sapConf.getLanguage();

        confLang = jQuery.sap.resources._normalizeLocale(confLang);
        // check if configured language is supported
        if(sapConf.getSupportedLanguages().indexOf(confLang) !== -1){
            sapConf.setLanguage(jQuery.sap.resources._convertLocaleToBCP47(confLang));
        } else {
            // check if only lang without sub script is supported.
            // like "fr_FR" might be configured language but not supported by app
            // but app supports "fr". Better using "fr" in such cases instead of "en-US"
            var lcl = new sap.ui.core.Locale(confLang);
            if(sapConf.getSupportedLanguages().indexOf(lcl.getLanguage()) !== -1){
                sapConf.setLanguage(lcl.getLanguage());
            } else { // fall back
                sapConf.setLanguage("en");
            }
        }
    };

    /**
     * Boots the sas commons infrastructure.
     * Creates a sas configuration object that can be accessed from sap.ui.getCore().getConfiguration() api.
     * Adds an ajax function to sas name space. A convenience api for Json Rest communication that supports cross domain calls using cross domain proxy.
     * Instantiates the WIP and Commons services registry if available. Exposed as sas.registry & sas.registry.wip respectively.
     * Provides hooks for consumers to register custom tasks to commons boot strap.
     * @param {function()} fnCallBack TODO-jsdoc
     */
    sas.hc.ui.core.Core.prototype.boot=function(fnCallBack){

        if(window.sas.hc.ui._bBooted){
            fnCallBack();
            return;
        }
        jQuery.sap.measure.start("sasCoreBootComplete");
        sas.hc.ui._bBooted = true;

        // populate sas configuration.
        var sasConf = new sas.hc.ui.core.Configuration();
        Object.defineProperty(sas,'config',{
            value : sasConf,
            enumerable:true,
        });

        registerCacheFilter();

        // create a sync point for tasks registration.
        var oHcSyncPoint = jQuery.sap.syncPoint("Sas commons bootstrap tasks",function(iOpenTasks, iFailures) {
                sas.log.trace("Sas commons bootstrap tasks registered: Total=" + iOpenTasks);
                jQuery.sap.measure.start("sasBootTaskExecution", "Core.js - boot");
                that.tasks.execute().then(function(){
                    that.tasks.destroy();
                    that.tasks = undefined;
                    jQuery.sap.measure.end("sasCoreBootComplete");
                    fnCallBack();
                });
            }),
            iCommonsBootTasks = oHcSyncPoint.startTask("sas commons boot tasks registration"),
            resolver=sas.hc.ui.core.util.ComponentServiceClassResolver,
            that=this,
            _isMainApplication = function (){
                return window === window.top;
            },
            regTask = resolver.getServiceClassInstance(resolver.REGISTRY_TASK),
            versionInfoTask = resolver.getServiceClassInstance(resolver.VERSION_INFO_TASK),
            cacheManagerTask = resolver.getServiceClassInstance(resolver.CACHE_MANAGER_TASK);


        // re-configure OpenUI5 language
        configureLanguage();

        // sas core util task.
        this._registerCoreTask(new sas.hc.ui.core.UtilTask());

        // CSRF token handler task
        this._registerCoreTask(new sas.hc.ui.core.CSRFTask());

        // setting info task
        this._registerCoreTask(new sas.hc.ui.core.settings.SettingsInfoTask());
        
        if(regTask){
            // Registry task if available
            this._registerCoreTask(regTask);
        }

        if (versionInfoTask) {
            sap.ui.getVersionInfo = versionInfoTask.getVersionInfo;
        }

        // user info
        this._registerCoreTask(new sas.hc.ui.core.util.UserInfoTask());

        if(cacheManagerTask){
            // register cache manager task.
            this._registerCoreTask(cacheManagerTask);
        }

        // check if app switcher is enabled and its a main window
        if(!sas.config.getApplicationSwitcherEnabled() ||  !_isMainApplication()){
            // don't execute these tasks in app switcher main window.
            // Talk to Yateen if you are not sure where should your task go.

            // theme root loader task.
            if(sasConf.getThemeUrl() !== undefined){
                this._registerCoreTask(new sas.hc.ui.core.theme.ThemeBootTask());
            }
            this._registerCoreTask(new sas.hc.ui.core.TouchDetectionTask());
            this._registerCoreTask(new sas.hc.ui.core.MonkeyPatchApplySettingsTask());
            this._registerCoreTask(new sas.hc.ui.core.ContextMenuTask());
        }

        // hook to register custom tasks to this sync point.
        var fnCustomBootTask = sasConf.getBootTask();
        if ( fnCustomBootTask ) {
            var iCustomBootTask = oHcSyncPoint.startTask("custom sas boot task");
            fnCustomBootTask( function(bSuccess) {
                oHcSyncPoint.finishTask(iCustomBootTask, typeof bSuccess === "undefined" || bSuccess === true );
            });
        }

        //register clearFocusRingTask for mouse and keyboard focus handling
        this._registerCoreTask(new sas.hc.ui.core.ClearFocusRingTask());

        // let session time out be the last task.
        this._registerCoreTask(new sas.hc.ui.core.TimeoutHandlerTask());

        // This comes after timeouts since app switcher needs to coordinate timeouts across apps and is dependent
        // on single app timeout already being in place
        if(sas.config.getApplicationSwitcherEnabled()===true)
            this._registerCoreTask(new sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherBootTask());

        oHcSyncPoint.finishTask(iCommonsBootTasks);
    };
    /**
     *
     * Registers a task to the core. The task needs to have run function on to it.
     * @param {object} oTask TODO-jsdoc
     */
    sas.hc.ui.core.Core.prototype.registerTask=function(oTask){
        if(!Array.isArray(oTask.dependsOn)){
            oTask.dependsOn = ["sas.hc.ui.core.RegistryTask","sas.hc.ui.core.theme.ThemeBootTask"];
        }
        this.tasks.add(oTask);
    };
}());

(function(){
    new sas.hc.ui.core.Core();
}());
