jQuery.sap.declare("sas.hc.ui.core.UtilTask");

jQuery.sap.require("sas.hc.ui.core.BootTask");
jQuery.sap.require("sas.hc.ui.core.GlobalSettings");
jQuery.sap.require("sas.hc.ui.core.util");
jQuery.sap.require("sas.hc.ui.core.util.ObjectProxy");
jQuery.sap.require("sas.hc.ui.core.util.jqXHRRetryHandler");
jQuery.sap.require("sap.ui.core.EventBus");
jQuery.sap.require("sas.hc.ui.core.theme.ThemeUtil");
jQuery.sap.require("sas.hc.ui.core.util.ServicesUtil");
(function(){
    sas.hc.ui.core.BootTask.extend('sas.hc.ui.core.UtilTask',{
        name: 'sas.hc.ui.core.UtilTask'
    });

    var usrPrefFormatLangs;

    function registerLanguageFilters(){
        var sapConf = sap.ui.getCore().getConfiguration();

        // set user preferred language on each http call.
        // do it asap.
        // http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.4
        // Using private API from jQuery.sap.resources to get appropriate user locale and fall back locale if
        // user locale happens to be not among supported locales by this application.
        // Expect private API to break when SDK is updated and this API gets dropped.
        // As per 15w33 image SupportedLanguages locales are populated in java.util.Locales format hence need to use this API.
        var usrPrefLangs = jQuery.sap.resources._getFallbackLocales(sapConf.getLanguage(),sapConf.getSupportedLanguages());
        if(Array.isArray(usrPrefLangs)){
            usrPrefLangs = usrPrefLangs.toString();
            if(usrPrefLangs.charAt(usrPrefLangs.length-1) === ','){
                // get rid of the last comma.
                usrPrefLangs = usrPrefLangs.substr(0,usrPrefLangs.length-1);
            }
            // SAP API returns locales in non BCP-47 syntax. Convert them.
            usrPrefLangs = usrPrefLangs.replace(/_/g,'-');
        }
        // defer 'Accept-Language' header value to browser if condition not met.
        if(typeof usrPrefLangs === 'string' && usrPrefLangs !== ''){
            var acceptLanguageAjaxFilter = function(options, originalOptions, jqXHR ) {
                if(typeof options !== 'object' || options === null){
                    // should never reach here. Someone trying to be innovative.
                    return;
                }
                options.headers = options.headers || {};
                if(typeof options.headers === 'object'){
                    var isAcceptLanguage = true;
                    var isAcceptFormatLanguage = true;
                    for(var prop in options.headers){
                        // better to do nothing if headers already has
                        // accept-language value be it in different case.
                        // jQuery.ajax#setRequestHeaders function normalizes header names.
                        if(prop.toLowerCase() === 'accept-language'){
                            isAcceptLanguage = false;
                        }
                        if(prop.toLowerCase() === 'accept-locale'){
                            isAcceptFormatLanguage = false;
                        }
                    }
                    if (isAcceptLanguage)
                        options.headers["Accept-Language"] = usrPrefLangs;
                    if (isAcceptFormatLanguage &&   // Accept-Format-Language request header
                        options.acceptLocale === true && // acceptLocale:true is explicitly specified in options?
                        usrPrefFormatLangs && typeof usrPrefFormatLangs === 'string' && usrPrefFormatLangs !== ''){
                        options.headers["Accept-Locale"] = usrPrefFormatLangs;
                    }
                }
            };

            jQuery.ajaxPrefilter(acceptLanguageAjaxFilter);
        }else{
            sas.log.error('Could not activate acceptLanguageAjaxFilter.');
        }
    };

    function registerFormatLocaleFilters(){
        var sapConf = sap.ui.getCore().getConfiguration();
        var formatLocale = sapConf.getFormatLocale();
        formatLocale = formatLocale.replace(/_/g,'-').toLowerCase(); // Convert to BCP 47 language tag.
        // Get preferred format locale and Set it in the OpenUI formatLocale
        sas.ajax({url:"/preferences/preferences/@currentUser/FormatLocale.DefaultLocale", async:true}).done(
            function(objUsrInfo){
                if(objUsrInfo && typeof objUsrInfo.value === 'string') {
                    formatLocale = objUsrInfo.value.replace(/_/g,'-').toLowerCase();// Convert to BCP 47 language tag.
                    sapConf.setFormatLocale(formatLocale);
                }
                updateFormatLocale(formatLocale);
            }).fail(function(oObj,txt){
                if(sas.log.info){
                    sas.log.info('sas.hc.ui.core.UtilTask','Could not fetch preferred format locale.');
                } else if(console && console.info){ // eslint-disable-line no-console
                    console.info('No preferred format locale found for current user. ' /* + oObj.statusText*/); // eslint-disable-line no-console
                }
                updateFormatLocale(formatLocale);
            });

        function updateFormatLocale(formatLocale) {
            // Update the usrPrefFormatLangs
            var aLocales = addOldJDKLocales(sas.hc.ui.core.GlobalSettings.SAS_SUPPORT_LOCALES);
            usrPrefFormatLangs = jQuery.sap.resources._getFallbackLocales(formatLocale, aLocales);
            if(Array.isArray(usrPrefFormatLangs)){
                usrPrefFormatLangs = usrPrefFormatLangs.toString();
                if(usrPrefFormatLangs.charAt(usrPrefFormatLangs.length-1) === ','){
                    // get rid of the last comma.
                    usrPrefFormatLangs = usrPrefFormatLangs.substr(0,usrPrefFormatLangs.length-1);
                }
                // SAP API returns locales in non BCP-47 syntax. Convert them.
                usrPrefFormatLangs = usrPrefFormatLangs.replace(/_/g,'-');
            }
        };
        // This regular expression came from jquery.sap.resources.js.
        var rLocale = /^((?:[A-Z]{2,3}(?:-[A-Z]{3}){0,3})|[A-Z]{4}|[A-Z]{5,8})(?:-([A-Z]{4}))?(?:-([A-Z]{2}|[0-9]{3}))?((?:-[0-9A-Z]{5,8}|-[0-9][0-9A-Z]{3})*)((?:-[0-9A-WYZ](?:-[0-9A-Z]{2,8})+)*)(?:-(X(?:-[0-9A-Z]{1,8})+))?$/i;
        function addOldJDKLocales(aLocales) {
            var M_ISO639_NEW_TO_OLD = {
                "he" : "iw",
                "yi" : "ji",
                "id" : "in",
                "sr" : "sh"
            };
            var aReturnLocales = [];
            aLocales.forEach(function(sLocale) {
                aReturnLocales.push(sLocale);
                var m = rLocale.exec(sLocale.replace(/_/g, '-'));
                var sLanguage = m[1].toLowerCase();
                sLanguage = M_ISO639_NEW_TO_OLD[sLanguage];
                if (sLanguage) {
                    var sScript = m[2] ? m[2].toLowerCase() : undefined;
                    var sRegion = m[3] ? m[3].toUpperCase() : undefined;
                    if ( sLanguage === "zh" && !sRegion ) {
                        if ( sScript === "hans" ) {
                            sRegion = "CN";
                        } else if ( sScript === "hant" ) {
                            sRegion = "TW";
                        }
                    }
                    aReturnLocales.push(sLanguage + ((sRegion) ? "-" + sRegion: ""));
                }
            });
            return aReturnLocales;
        };
    };

    // utility function used in sas.ajax'
    var shouldHandleAuthRetry;

    var logIfLocalAjaxEventsUsed = function(options){
        //http://api.jquery.com/Ajax_Events/
        if(options.success || options.error || options.complete){
            sas.log.error('[sas.ajax]','Use of local ajax events like success/error/complete is strongly discouraged as it hinders execution of built-in retry handlers.'
                        + ' Use promise callbacks like then/done/fail/always instead. See http://api.jquery.com/Types/#Promise','Url:'+options.url);
            // log error once and change the function to no-op to avoid
            // executing then onwards.
            logIfLocalAjaxEventsUsed = Function.prototype;
        }
    };

    function _invokeAjaxCall(options){
        var retObj = jQuery.ajax(options);
        // check for sas handleAuthentication option
        if ( shouldHandleAuthRetry === true) {
            logIfLocalAjaxEventsUsed(options);
            return new sas.hc.ui.core.util.ObjectProxy(
                retObj,
                new sas.hc.ui.core.util.jqXHRRetryHandler(retObj,options)
            );
        } else {
            return retObj;
        }
    }

    function defineSasAjax(){

        shouldHandleAuthRetry = sas.config.get('xx-sasAjaxHandleAuthenticationRetry') !== false;

        // add sas ajax function.
        Object.defineProperty(sas,'ajax', {
            enumerable:true,
            value: function(options){

                options = options || {};
                options.acceptLocale = options.acceptLocale !== false ? true : false;
                options.type = typeof options.type === 'string' ? options.type.toUpperCase() : 'GET';
                options.processData = typeof options.processData === 'boolean' ? options.processData : true;
                // default the contents to json.
                if(options.dataType === undefined){
                    options.dataType = 'json';
                }
                if(options.contentType === undefined){
                    options.contentType = 'application/json';
                    // jsonize data if JSON exist.
                    if(options.processData && window.JSON && options.type !== 'GET' && typeof options.data === 'object'){
                        options.data = JSON.stringify(options.data);
                    }

                }
                // check for sas specific options
                if((options.sas && options.sas.crossDomain) || sas.hc.ui.core.util.isUrlCrossDomain(options.url)){
                    // change the url
                    var q = {};
                    var destUrl = options.url;
                    if(options.data && options.type === 'GET' && options.processData){
                        var params = jQuery.param( options.data,options.traditional);
                        destUrl += ( /\?/.test( destUrl ) ? "&" : "?" ) + params;
                        delete options.data;
                    }
                    q['sas-commons-destUrl'] = destUrl;
                    // check for authentication.
                    if(options.sas && options.sas.authenticate){
                        q['sas-commons-auth'] = true;
                    }
                    // if cross domain proxy config value doesn't start or contain '/' (a case with local cdps)
                    // then treat it as running locally within application.
                    options.url = sas.hc.ui.core.util._getCrossDomainProxyUrl()+"?"+jQuery.param(q);
                }
                var retObj;
                // check if there is any cache registered to handle this request.
                if(sas.cacheManager){
                    var _cache = sas.cacheManager.findCache(options);
                    if(_cache){
                        retObj = _cache.handle(options,_invokeAjaxCall);
                    }
                }
                return retObj || _invokeAjaxCall(options);
            }
        });
    }

    function configureThemeRoot(){
        // patch up the theme roots, our version of the OpenUI SDK has a bug in sap.ui.core.Configuration that will mask the themeRoot property in the global configuration
        // we setup a global themeroots property on the configuration similar to the way it is handled in the 1.26 SDK
        // TODO when we upgrade to a more recent SDK, verify we didn't break anything and still need to wrap cross domain urls
        var config = window["sap-ui-config"] || {};
        var themeRoots = config.themeroots || {};
        var core = sap.ui.getCore();

        // setup the roots
        for (var theme in themeRoots) {
            if (themeRoots.hasOwnProperty(theme)) {
                var themeRoot = themeRoots[theme];
                if(typeof themeRoot === 'string'){

                    // determine value of unversionedThemes. GUIDESINF-1416
                    // should be true in non Petrichor platofrm unless specified otherwise.
                    var unversionedThemes = config.sas && config.sas.unversionedThemes;
                    if(typeof unversionedThemes === 'undefined'){
                        var servicePlatform = sas && sas.config && sas.config.get('servicePlatform');
                        if (servicePlatform === 'PETRICHOR'){
                            unversionedThemes = false;
                        } else {
                            unversionedThemes = true;
                        }
                    }

                    //allows applications to override the theme URL adding version information (currently used by QUnit and ThemeHost
                    if(unversionedThemes){
                        core.setThemeRoot(theme, themeRoot);
                    }else {
                        var themeUtil = new sas.hc.ui.core.theme.ThemeUtil();
                        var versionInfo = sap.ui.getVersionInfo();
                        if (versionInfo && versionInfo.libraries) {
                            versionInfo.libraries.forEach(function (library) {
                                core.setThemeRoot(theme, [library.name], themeRoot + themeUtil.getVersionLibraryThemeVersion(library.name, themeUtil.CURRENT_COMMONS_VERSION));
                            }, this);
                        }
                    }
                }else if(typeof themeRoot === 'object'){
                    for (var lib in themeRoot) {
                        if (lib.length > 0) {
                            core.setThemeRoot(theme, [lib], themeRoot[lib]);
                        } else {
                            core.setThemeRoot(theme, themeRoot[lib]);
                        }
                    }
                }
            }
        }
    }


    sas.hc.ui.core.UtilTask.prototype.run=function(){

        registerLanguageFilters();

        // add SAS application level EventBus.  Could return the one
        // from sap.ui.getCore() but create a dedicate SAS bus instead.
        Object.defineProperty(sas,'eventBus',{
            value : new sap.ui.core.EventBus(),
            enumerable:true,

        });

        defineSasAjax();

        if (sas.hc.ui.core.util.ServicesUtil.getServicePlatform() === sas.hc.ui.core.util.ServicesUtil.ServicePlatform.PETRICHOR)
            registerFormatLocaleFilters();

        configureThemeRoot();

        // register an even listener for ltjs loading if any
        sas.eventBus.subscribe("ltjs","loading_started",function(){
            // code to update splash screen
            var splashScreenHeaderElm = jQuery("#sas-hc-splash-headerText");
            if( splashScreenHeaderElm && splashScreenHeaderElm.length > 0){
                // splash screen found.
                var rb = sap.ui.getCore().getLibraryResourceBundle("sas.hc.ui.core");
                splashScreenHeaderElm.text(rb && rb.getText('splash.header.initializing.txt'));
            }
        });

        if (sap.ui.Device.browser.edge) {
            jQuery("html").attr("x-ms-format-detection", "none");
        }
    };
}());